"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildSchema = exports.scalarResolvers = void 0;
const fs_1 = __importDefault(require("fs"));
const apollo_error_1 = require("apollo-error");
const schema_1 = require("@graphql-tools/schema");
const delegate_1 = require("@graphql-tools/delegate");
const p_retry_1 = __importDefault(require("p-retry"));
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("@cardano-graphql/util"));
const buildHasuraSchema_1 = require("./buildHasuraSchema");
const DB_1 = require("./DB");
const GraphQLBigInt = require('graphql-bigint');
exports.scalarResolvers = {
    Hash32HexString: util_1.default.scalars.Hash32HexString,
    BigInt: GraphQLBigInt,
    DateTime: util_1.default.scalars.DateTimeUtcToIso,
    Percentage: util_1.default.scalars.Percentage
};
function buildSchema(hasuraUri) {
    return __awaiter(this, void 0, void 0, function* () {
        let hasuraSchema;
        yield p_retry_1.default(() => __awaiter(this, void 0, void 0, function* () {
            hasuraSchema = yield buildHasuraSchema_1.buildHasuraSchema(hasuraUri);
        }), {
            factor: 1.75,
            retries: 9,
            onFailedAttempt: util_1.default.onFailedAttemptFor('Fetching Hasura schema via introspection')
        });
        const db = new DB_1.DB(hasuraUri);
        return schema_1.makeExecutableSchema({
            resolvers: Object.assign({}, exports.scalarResolvers, {
                Query: {
                    cardanoDbMeta: () => __awaiter(this, void 0, void 0, function* () {
                        try {
                            return db.getMeta();
                        }
                        catch (error) {
                            throw new apollo_error_1.ApolloError(error);
                        }
                    }),
                    blocks: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'blocks',
                            info,
                            operation: 'query',
                            schema: hasuraSchema
                        });
                    },
                    blocks_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'blocks_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraSchema
                        });
                    },
                    epochs: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'epochs',
                            info,
                            operation: 'query',
                            schema: hasuraSchema
                        });
                    },
                    epochs_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'epochs_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraSchema
                        });
                    },
                    cardano: (_root, _args, context, info) => __awaiter(this, void 0, void 0, function* () {
                        const result = (yield delegate_1.delegateToSchema({
                            context,
                            fieldName: 'cardano',
                            info,
                            operation: 'query',
                            schema: hasuraSchema
                        }))[0];
                        if (result.currentEpoch === null) {
                            return new apollo_error_1.ApolloError('currentEpoch is only available when close to the chain tip. This is expected during the initial chain-sync.');
                        }
                        return result;
                    }),
                    transactions: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'transactions',
                            info,
                            operation: 'query',
                            schema: hasuraSchema
                        });
                    },
                    transactions_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'transactions_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraSchema
                        });
                    },
                    utxos: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'utxos',
                            info,
                            operation: 'query',
                            schema: hasuraSchema
                        });
                    },
                    utxos_aggregate: (_root, args, context, info) => {
                        return delegate_1.delegateToSchema({
                            args,
                            context,
                            fieldName: 'utxos_aggregate',
                            info,
                            operation: 'query',
                            schema: hasuraSchema
                        });
                    }
                }
            }),
            typeDefs: fs_1.default.readFileSync(path_1.default.resolve(__dirname, '..', 'schema.graphql'), 'utf-8')
        });
    });
}
exports.buildSchema = buildSchema;
//# sourceMappingURL=executableSchema.js.map